import urllib


class Config:
    IMAGE_SEARCH_URL = "https://tr.pinterest.com/resource/BaseSearchResource/get/?"
    def __init__(self, search_keywords="", file_lengths=100, image_quality="orig", bookmarks="", scroll=0):
        self.search_keywords = search_keywords
        self.file_lengths = file_lengths
        self.image_quality = image_quality
        self.bookmarks = bookmarks
        self.scroll = str(scroll)

    #image search url
    @property
    def search_url(self):
        return self.IMAGE_SEARCH_URL

    #search parameter "source_url"
    @property
    def source_url(self):
         return "/search/pins/?q=" + urllib.parse.quote(self.search_keyword)

    #search parameter "data"
    @property
    def image_data(self):
        # if self.bookmarks == "":
        #     return '''{"options":{"page_size":250, "isPrefetch":true, "scroll":''' + self.scroll + ''', "query":"''' + self.search_keyword + '''","scope":"pins","no_fetch_context_on_resource":false},"context":{}}'''
        # else:
            return '''{"options":{"page_size":250, "scroll":''' + self.scroll + ''', "query":"''' + self.search_keyword + '''","scope":"pins","bookmarks":["''' + self.bookmark + '''"],"field_set_key":"unauth_react","no_fetch_context_on_resource":false},"context":{}}'''.strip()

    @property
    def search_keyword(self):
        return self.search_keywords
    
    @search_keyword.setter
    def search_keyword(self, search_keywords):
        self.search_keywords = search_keywords
    
    @property
    def file_length(self):
        return self.file_lengths
    
    @file_length.setter
    def file_length(self, file_lengths):
        self.file_lengths = file_lengths

    @property
    def image_quality(self):
        return self.image_qualitys
    
    @image_quality.setter
    def image_quality(self, image_qualitys):
        self.image_qualitys = image_qualitys

    @property
    def bookmark(self):
        return self.bookmarks

    @bookmark.setter
    def bookmark(self, bookmarks):
        self.bookmarks = bookmarks
    


import time
import subprocess


while True:
    command = "python main.py"
    process = subprocess.Popen(command, shell=True)
    process.wait()
    print("Sleeping for 30 seconds ...")
    time.sleep(30)



import os
import random
import argparse
from itertools import combinations
from .scraper import Scraper
from .config import Config


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--keywords", help="keywords as path of file or list of words", nargs='+', default=[])
    parser.add_argument('--output', help='output dir', default='./io/output', type=str)
    parser.add_argument("-nw", '--number-of-words', help='number of keywords for search', default=2, type=int)
    args = parser.parse_args()

    os.makedirs(args.output, exist_ok=True)
    keywords = []
    for keyword in args.keywords:
        if os.path.isfile(keyword):
            file = open(keyword, "r", encoding="utf-8")
            keywords += [keyword.strip() for keyword in file.read().split('\n')]
        elif isinstance(keyword, str):
            keywords.append(keyword)
    random.shuffle(keywords)
    
    print("start crawling...")
    counter = 0
    if len(keywords) == 1:
        args.number_of_words = 1
    for item in combinations(keywords, args.number_of_words):
        if counter == 4:
            break

        keyword = " ".join(word for word in item)
        print(keyword)

        while True:
            configs = Config(search_keywords=keyword,  # Search word
                                      # total number of images to download (default = "100")
                                      file_lengths=5000,
                                      # image quality (default = "orig")
                                      image_quality="originals",
                                      # next page data (default= "")
                                      bookmarks="",
                                      scroll=10000)

            # download images directly
            number = Scraper(configs).download_images(args.output)  
            print("number:", number)
            if number == 0:
                counter += 1
                break
            else:
                counter = 0

    print("All images in dir:", len(os.listdir(args.output)))


if __name__ == "__main__":
    main()



from pathlib import Path
from setuptools import setup, find_packages


def post_install():
    """ Implement post installation routine """
    with open('./requirements.txt') as f:
        install_requires = f.read().splitlines()

    return install_requires


def pre_install():
    """ Implement pre installation routine """
    # read the contents of your README file
    global long_description
    this_directory = Path(__file__).parent
    long_description = (this_directory / "README.md").read_text()


pre_install()


setup(
    name='pinterest-crawler',
    version='0.1.1',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=["pinterest_crawler"],
    setup_requires=[],
    url='https://github.com/SajjadAemmi/Pinterest-Crawler',
    license='',
    author='Sajjad Aemmi',
    author_email='sajjadaemmi@gmail.com',
    description='Pinterest Crawler: Download as many images as you want about the searched words',
    include_package_data=True,
    install_requires=post_install(),
    entry_points={
        "console_scripts": ["pinterest-crawler=pinterest_crawler.pinterest_crawler:main"],
    },
)



import os
import json
import urllib
import requests


class Scraper:
    def __init__(self, config, image_urls=None):
        self.config = config
        self.image_urls = image_urls if image_urls else []

    def setConfig(self, config):
        # Set config for bookmarks (next page)
        self.config = config

    def download_images(self, output_path) -> int:
        # prev get links
        results = self.get_urls()
        try:
            os.makedirs(output_path)
            print("Directory ", output_path, " Created ")
        except FileExistsError:
            pass
        
        number = 0
        listdir = os.listdir(output_path)
        if results != None:
            for i in results:
                file_name = i.split("/")[-1]
                if file_name not in listdir:
                    try:
                        number += 1
                        print("Download ::: ", i)
                        urllib.request.urlretrieve(i, os.path.join(output_path, file_name))
                    except Exception as e:
                        print("Error:", e)

        return number

    def get_urls(self) -> list:
        SOURCE_URL = self.config.source_url,
        DATA = self.config.image_data,
        URL_CONSTANT = self.config.search_url

        r = requests.get(URL_CONSTANT, params={"source_url": SOURCE_URL, "data": DATA})
        jsonData = json.loads(r.content)
        resource_response = jsonData["resource_response"]
        data = resource_response["data"]
        results = data["results"]
        
        for i in results:
            try:
                self.image_urls.append(i["objects"][0]["cover_images"][0]["originals"]["url"])
            except:
                self.URL = None
                self.search(i)
                if self.URL != None:
                    self.image_urls.append(self.URL)

        if len(self.image_urls) < int(self.config.file_length):
            try:
                print("Creating links", len(self.image_urls))
                return self.image_urls[0:self.config.file_length]
            except:
                pass

    def search(self, d):
        if isinstance(d, dict):
            for k, v in d.items():
                if isinstance(v, dict):
                    if k == "orig":
                        self.URL = v["url"]
                    else:
                        self.search(v)
                elif isinstance(v, list):
                    for item in v:
                        self.search(item)
